"""Class to query database info

This is MySQL specific.

OTHER DBs:
  1. Create a file in PSQLInput/DBInfo named with your database connection
     "database_type" attribute
  2. Create a class called Info that implements the methods below

It shouldn't be difficult to adapt to other databases. Probably the best way
to do it is to it is to inherit from this class. I've tried to note which 
functions are really MySQL specific and should be overloaded.

Mail me if you adapt for other DB (I just have access to Mysql):

Author: Paulo Eduardo Neves <neves@inf.puc-rio.br>
"""

import string

class Info:
    "subclasses must fill with missed info and functions"
    queryTables = '' #fill with db command to get tables names, e.g. 'show table'
    queryColumns = '' #fill with db command to get columns names from a table, e.g. 'show columns from %s' 

    def __init__(self, connection):
	self.connection = connection
	self.columnsData = {}
	self.tablesData = []

    def query(self, command):
	dummy, result = self.connection().query(command)
	return  result


    def getTablesNames(self):
	"""Gets the answer of the query of tables names 
	and returns a list with the tables names. 

	Uses the `queryTables' SQL command to produce a list of table names"""

	self.tablesData = self.query(self.queryTables)
	#This is a list where each element is a unitary list with the table name
	#let's make it a simple list:
	self.tablesData = map(lambda x: x[0], self.tablesData)
	return self.tablesData

    def getColumnsNames(self, tables):
	"""Query the columns names and returns a dictionary
	where the key is the table name and the value is the columns list"""

	tableColumns = {}
	for i in tables:
	    self.columnsData[i] = self.query(self.queryColumns % i)
	    columns = []
	    for j in self.columnsData[i]:
		if not self._identitycol(j): #needed by some DBs
		    columns.append(self._columnName(j))
	    tableColumns[i] =  columns

	return tableColumns

    def type(self, table, column):
	"""Maps database types to #sqlvar types

	Subclasses MUST implement it
	"""
	pass
    
    def nullable(self, table, column):
	"""Says if a column can have a NULL value, if yes returns 'optional"

	Subclasses MUST implement it
	"""
	pass

    def _columnName(self, columnData):
	"""Chooses the field in database query that is the column name"

	Subclasses SHOULD implement it
	"""

	return columnData[0] #is the first column


    def _identitycol(self, columnData):
	"Some DBs (Sybase) need this function"
	return 0

