"""Class to query database info

This is Interbase specific. v0.3

Author: Tom Deprez <tom.deprez@uz.kuleuven.ac.be>

History:
--------

0.3 : converted DEFAULT values to their string representation if needed, 
      otherwise to None.(thanks to bob.tierney@goingv.com for explanation 
      the way gvib works with Blob fields and pointing me to this)        
0.2 : converted DEFAULT values to None, to prevent error
      with some default types
0.1 : first release             
"""

import InfoBase
import string
import copy 

class Info(InfoBase.Info):
    # We've to use the following queries to overcome the 
    # DSQL restrictions in Interbase     
    queryTables = """ Select RDB$RELATION_NAME AS TableName
                      From RDB$RELATIONS                      
                      Where RDB$RELATION_NAME NOT Like 'RDB$%'
                      Order By RDB$RELATION_NAME"""           

    queryColumns = """ Select A.RDB$FIELD_NAME AS Fld,                        
                              C.RDB$TYPE_NAME AS Typ,
                              B.RDB$FIELD_LENGTH AS Len,
                              A.RDB$NULL_FLAG AS N_Nul,
                              A.RDB$DEFAULT_SOURCE AS Def,
                             -B.RDB$FIELD_SCALE AS Scale
                       From RDB$RELATION_FIELDS A
                       left join RDB$FIELDS B
                         on A.RDB$FIELD_SOURCE=B.RDB$FIELD_NAME
                       left join RDB$TYPES C
                         on B.RDB$FIELD_TYPE=C.RDB$TYPE
                       Where A.RDB$RELATION_NAME='%s'
                         and C.RDB$FIELD_NAME='RDB$FIELD_TYPE'                                              
                       ORDER BY RDB$FIELD_POSITION""" 

    def _columnName(self, columnData):
	"""Chooses the field in database query that is the column name"

	Subclasses SHOULD implement it
	"""

	return string.rstrip(columnData[0]) #is the first column
    
    def getTablesNames(self):
	"""Gets the answer of the query of tables names 
	and returns a list with the tables names. 

	Uses the `queryTables' SQL command to produce a list of table names"""

	self.tablesData = self.query(self.queryTables)
	#This is a list where each element is a unitary list with the table name
	#let's make it a simple list: 
        #Interbase specific : strip trailing spaces
	self.tablesData = map(lambda x: string.rstrip(x[0]), self.tablesData)
	return self.tablesData

    def getColumnsNames(self, tables):
	"""Query the columns names and returns a dictionary
	where the key is the table name and the value is the columns list"""

	tableColumns = {}
	for i in tables: 
            self.columnsData[i] = self.query(self.queryColumns % i)
	    columns = []
	    for j in self.columnsData[i]:
              """ 
                 Interbase specific : 
                    strip trailing spaces
                    convert Python buffer objects to their string
		      presentation, if needed               
              """ 
              tmpColumnsData = (string.rstrip(j[0]),)+(string.rstrip(j[1]),) + j[2:4]
              if j[4]==None:               
                tmpColumnsData = tmpColumnsData + (None,)           
              else:                    
                tmpColumnsData = tmpColumnsData + (str(j[4]),)              
              tmpColumnsData = tmpColumnsData + j[5:]                                 
              self.columnsData[i][self.columnsData[i].index(j)]=tmpColumnsData 

	      if not self._identitycol(j): #needed by some DBs
                columns.append(self._columnName(j)) 	    
            tableColumns[i] =  columns        

        return tableColumns

    def type(self, table, column):        
	#Maps database types to sqlvar types
        for i in self.columnsData[table]:
	    if i[0] == column:
                colInfo = i
		break
	# in `queryColumns', second col is type
	type = string.lower(colInfo[1])                      
        if ((string.find(type, "date") != -1) or  
            (string.find(type, "time") != -1) or      #gets time &timestamp             
            (string.find(type, "text") != -1) or
            (string.find(type, "varying") != -1) or
            (string.find(type, "cstring") != -1) or
            (string.find(type, "char") != -1) ):      #gets char & varchar 
    
            if self.nullable(table, column): 
		#I'm assuming that if a string field can be null,
		#there's no sense for it to be empty
		return 'nb'
	    else:
		return 'string'       
        
        
        if ((string.find(type, "short") != -1) or       
            (string.find(type, "long") != -1) or      
            (string.find(type, "quad") != -1) or
            (string.find(type, "blob_id") != -1) or
            (string.find(type, "smallint") != -1) or
            (string.find(type, "integer") != -1) or
            (string.find(type, "int64") != -1) ):       
    
            return 'int'
        

        if ((string.find(type, "float") != -1) or     #gets float & d_float  
            (string.find(type, "double") != -1) ):

            return 'float' 

    def nullable(self, table, column):
	"This is really Interbase specific"
	for i in self.columnsData[table]:
	    if i[0] == column:
		colInfo = i
		break
	if colInfo[3] == 0: #3 is the pos for null info
	    return 'optional'
	return ''


