"""Class to query database info

This is ZODBC specific.

Author: Paulo Eduardo Neves <neves@inf.puc-rio.br>
        Philipp Auersperg<phil@bluedynamics.com>
"""

import InfoBase
import string
from string import join, split, find, strip

class Info(InfoBase.Info):
    #Are these commands in the SQL standard?
    queryTables = 'show tables'
    queryColumns = 'show columns from %s' 

    def type(self, table, column):
	"""Maps database types to #sqlvar types

	"""
	for i in self.columnsData[table]:
	    if i[0] == column:
		colInfo = i
		break
	type = string.lower(colInfo[1]) #[1] is the pos for the type info
	if ((string.find(type, "char") != -1) or #gets char and varchar
	    (string.find(type, "set")  != -1) or 
	    (string.find(type, "text") != -1) or
	    (string.find(type, "date") != -1) or
	    (string.find(type, "time") != -1) or
	    (string.find(type, "enum") != -1) ):

	    if self.nullable(table, column): 
		#I'm assuming that if a string field can be null,
		#there's no sense for it to be empty
		return 'nb'
	    else:
		return 'string'
	if (string.find(type, "int") != -1):
	    return 'int'
	
	#else
	return 'float'
    
    def nullable(self, table, column):
	"This is really MySQL specific"
	for i in self.columnsData[table]:
	    if i[0] == column:
		colInfo = i
		break
	if colInfo[2] == "with Null": #2 is the pos for null info
	    return 'optional'
	return ''

    def getTablesNames(self):
#	print 'ODBC::getTableNames 1'
	self.tablesData=[]
	
	rs = self.connection().tables()
#	print 'ODBC::getTableNames 2'
	tabs=split(rs,'\n')
	for cur in tabs:
#		print ['tab:' , cur]
		row=split(cur,'\t')
		try:
			self.tablesData.append(row[2])
		except:
			pass

		#print 'tab='+row[2]

	#self.tablesData = map(lambda x: x[0], self.tablesData)

		
	return  self.tablesData

    def getColumnsNames(self, tables):
	"""Query the columns names and returns a dictionary
	where the key is the table name and the value is the columns list"""

#	print 'getColumnNames: '
	print  tables

	tableColumns = {}
	colspertab=[]

#	print 'getColumnNames 1'
	
	for i in tables:
#		print 'getColumnNames 2'
	    	cols=self.connection().columns(i)
#		print 'getColumnNames 3'
		
		for col in cols:
			line=[col['Name'],col['Type'],'%d' % (col['Precision']) ,col['Nullable']]
			colspertab.append(line)
	
		
	    	self.columnsData[i] = colspertab
	    	columns = []
	    	for j in self.columnsData[i]:
			if not self._identitycol(j): #needed by some DBs
		    		columns.append(self._columnName(j))

#		print 'columns:'
#		print columns
		tableColumns[i] =  columns


#		print 'self.columnsData:'
#		print self.columnsData

#		print 'tableColumns:'
#		print tableColumns

	return tableColumns

