
"""Class to query database info

This is PostgreSQL specific.

Author: Paulo Eduardo Neves <neves@inf.puc-rio.br>

PostgreSQL Modifications: Cary O'Brien <cobrien@radix.net> 
    (Tested with PostgreSQL 6.4/Linux/glibc/Python 1.5.1)
    PostgreSQL interface is 1) missing nullable, 2) not sure
    it checks for all datatypes.

"""

import InfoBase
import string

class Info(InfoBase.Info):
    #Are these commands in the SQL standard?
    #queryTables = 'show tables'
    #queryColumns = 'show columns from %s' 

    #Postgresql versions of the queries.  Ugly, eh?

    #PHIL: changed keywords to lowercase, otherwise PoPyDA feels inconvenient (returns empty recordsets)
    queryTables = 'select relname \
                   from pg_class, pg_user \
                   where  relkind = \'r\'    \
                   and relname !~ \'^pg_\'  and relname !~ \'^xin[vx][0-9]+\' \
                   and usesysid = relowner order by relname'

    queryColumns = 'select  a.attnum, a.attname, t.typname, a.attlen, \
                   a.atttypmod, a.attnotnull, a.atthasdef \
                   from pg_class c, pg_attribute a, pg_type t \
                   where c.relname = \'%s\'    \
                     and a.attnum > 0   \
                     and a.attrelid = c.oid   \
                     and a.atttypid = t.oid  \
                    order by attnum'

    def type(self, table, column):
        """Maps database types to #sqlvar types

        This looks like really PGSQL specific
        """
        for i in self.columnsData[table]:
            if i[1] == column:
                colInfo = i
                break
        type = string.lower(colInfo[2]) #[2] is the pos for the type info
        if ((string.find(type, "char") != -1) or #gets char and varchar
            (string.find(type, "text") != -1) or
            (string.find(type, "date") != -1) or
            (string.find(type, "time") != -1) or
            (string.find(type, "bool") != -1) or
            (string.find(type, "enum") != -1) ):

            if self.nullable(table, column): 
                #I'm assuming that if a string field can be null,
                #there's no sense for it to be empty
                return 'nb'
            else:
                return 'string'
        if (string.find(type, "int") != -1):
            return 'int'
        
        #else
        return 'float'
    
    def nullable(self, table, column):
        #
        # I couldn't get this to work.  I didn't
        # try very hard.
        #   -- Cary
        #
        #for i in self.columnsData[table]:
        #    if i[0] == column:
        #       colInfo = i
        #       break
        #if self.colInfo[5] == 't': #5 is the pos for null info
        #    return 'optional'
        return ''


    def _columnName(self, columnData):
        """Chooses the field in database query that is the column name"

        Subclasses SHOULD implement it
        """

        return columnData[1] #is the second column
