"""Class to query database info

This is Sybase/MS SQL specific.

It's been tested with MS SQL Server 6.5

Author: Anthony Baxter <anthony@interlink.com.au>
"""

import InfoBase
import string

class Info(InfoBase.Info):
    # mmm... standards.
    queryTables = "select name from sysobjects where type = 'U'"
    # could use sp_columns - but not sure it's going to work nicely
    # with the db-sig api.
    queryColumns = """ select syscolumns.name as name,
                              systypes.name as type,
                              syscolumns.length as length,
                              syscolumns.status as status
                       from syscolumns,systypes,sysobjects
                       where sysobjects.type = 'U'
                             and sysobjects.name = '%s'
                             and syscolumns.id = sysobjects.id
                             and syscolumns.usertype = systypes.usertype
                             and systypes.name != 'sysname' """


    def type(self, table, column):
	"""Maps database types to #sqlvar types
	"""
	for i in self.columnsData[table]:
	    if i[0] == column:
		colInfo = i
		break
	# in `queryColumns', second col is type
	type = string.lower(colInfo[1]) 
	if ((string.find(type, "char") != -1) or  #gets char and varchar
	    (string.find(type, "datetime") != -1) or
	    (string.find(type, "text") != -1) or
	    (string.find(type, "smalldatetime") != -1) ):
	    if self.nullable(table, column):
		return 'nb'
	    else:
		return 'string'
	if ( (string.find(type, "rowid") != -1) or
	    (string.find(type, "numeric") != -1) or
	    (string.find(type, "decimal") != -1) or
	    (string.find(type, "int") != -1) or
	    (string.find(type, "smallint") != -1) or
	    (string.find(type, "tinyint") != -1) or
	    (string.find(type, "number") != -1) ):
	    return 'int'
	#else
	return 'float'
    
    def nullable(self, table, column):
	"This is really Sybase specific. Sybase blows goats."
	for i in self.columnsData[table]:
	    if i[0] == column:
		colInfo = i
		break
	if colInfo[3] & 0x08: #3 is the pos for null info, 0x08 is the mask
                              # saying 'null ok'
	    return 'optional'
	return ''


    def _identitycol(self, columnData):
	"This is really Sybase specific"
	return columnData[3]&0x80
