"""Znolk SQL Input Wizard

Help to create method, form and result page to insert data in a database.

This is a wizard to help create database input forms and SQL methods. 
The idea is to help the user to select the database tables and columns to 
which input data and the wizard automagically creates the insert SQL 
method and the HTML input form.

You'll probably have to edit the generated documents and method. The method 
specially if you are inputing data in multiple tables, since you'll want to 
have the same data in multiple dates (e.g. person_id).
Remember that it is easier to take out paramenters than to search your 
documentation/database for them, so don't be shy selecting items.

This software is released under GNU public license. See details in the URL:
http://www.gnu.org/copyleft/gpl.html

Author: Paulo Eduardo Neves <neves@inf.puc-rio.br>

New Features (Update,Delete,Browse,support for ODBC Databases) added by:

Philipp Auersperg <phil@BlueDynamics.com>

www.bluedynamics.com

Please send me bugs and feedback about how to improve it.
"""
import string
import Acquisition
from Globals import HTMLFile, Persistent, MessageDialog
from  Products.ZSQLMethods.SQL import *



def implementedSQLConnectionIDs(self):
    """Find SQL database connections in the current folder and above

    This function return a list of ids.
    Shameless stolen (with little modifications) from SQL.py
    """
    ids={}
    have_id=ids.has_key
    StringType=type('')

    while self is not None:
        if hasattr(self, 'objectValues'):
            for o in self.objectValues():
		try:
		    if (hasattr(o,'_isAnSQLConnection') 
			and o._isAnSQLConnection
			and hasattr(o,'id')):
			#let's see if this DB is suported
			if not hasattr(o, 'database_type'):
			    raise ImportError
			exec('import DBInfo.' + getattr(o, 'database_type'))
			
			id=o.id
			if type(id) is not StringType: id=id()
			if not have_id(id):
			    if hasattr(o,'title_and_id'): o=o.title_and_id()
			    else: o=id
			    ids[id]=id
		except ImportError:
		    #Database is not supported
		    pass 
        if hasattr(self, 'aq_parent'): self=self.aq_parent
        else: self=None

    ids=map(lambda item: (item[1], item[0]), ids.items())
    ids.sort()
    return ids

manage_startWizard = HTMLFile('selectDB', globals())

def _generateId(self, baseName):
    "Returns a not used attribute in the self object"
    id = baseName
    count = 1
    while hasattr(self, id):
	id = baseName + `count`
	count = count + 1
    return id

def debug(self):
    "debug"
    return ""

def manage_createWizard(self, connection_id, REQUEST):
    """Create the wizard for the desired connection_id
       and call it's default method """

    id = _generateId(self, "psqlwiz") #default name
    wiz = ZnolkSQLWizard(id, connection_id)
    setattr(self, id, wiz)

    REQUEST.RESPONSE.redirect("%s/%s" % (REQUEST['URL1'], id))

class Table:
    """Class to group table and columns atributtes
    Just to have a cleaner (dtml) code"""
#PHIL:orig    def __init__(self, name, columns, optionalColumns = []):
    def __init__(self, name, columns,optionalColumns = [],pkColumns=[],skipPKs=0,controls=None,extensions=None,allColumns=None):
	self.tableName = name
	self.columns = columns
	self.optional = optionalColumns
	self.pkColumns=pkColumns #PHIL
	self.skipPKs=skipPKs #PHIL:skip PKs from insert statement
	#PHIL
        self.columnControls={}
        #print '------------------'
        #print 'COLUMNS:',columns
        #print 'controls:',controls
        
        #print '------------------'
        if controls != None:
            for i in range(0,len(allColumns)):
                if allColumns[i] in columns: #weird but must be
                    self.columnControls[allColumns[i]]=[controls[i],extensions[i]]
                #print i
            #print 'columnControls:'     ,self.columnControls,'COLUMNS:',self.columns
	#/PHIL
#	print 'Table::init:',name,pkColumns,optionalColumns,skipPKs


#        if controls != None:
#            for i in range(0,len(columns)):
#                self.columnControls[columns[i]]=[controls[i],extensions[i]]
#                #print i
#            print 'columnControls:'     ,self.columnControls,'COLUMNS:',self.columns
	#/PHIL
#	print 'Table::init:',name,pkColumns,optionalColumns,skipPKs
	
    def __str__(self):
	return "Table Name:%s<BR>\nColumns:%s" % (self.tableName, `self.columns`)

class ZnolkSQLWizard(
    Acquisition.Implicit,
    #Persistent,
    ):
    """A Wizard to create  forms and methods to insert data in databases

    The wizard uses a sequence of forms to ask the user for data,
    these data are stored as attributes in the wizard. When all the necessary 
    data are gathered it creates the input form, result page and method.
    """
    inputBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><INPUT TYPE="TEXT" NAME="%s" value="<dtml-var %s null="">"></TD></TR>'
    insert_inputBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><INPUT TYPE="TEXT" NAME="%s"></TD></TR>'
    textAreaBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><textarea NAME="%s" cols=40 rows=4 ><dtml-var %s null=""></textarea></TD></TR>'
    insert_textAreaBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><textarea NAME="%s" cols=40 rows=4 ></textarea></TD></TR>'
    labelBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><dtml-var %s><input type="hidden" name=%s value="<dtml-var %s null="">"</TD></TR>' #PHIL for primary keys

    booleanBase = '''\t<TR>
<TD ALIGN="RIGHT">%s</TD>
   <TD>
      <select NAME="%s">
         <option value="0"
           <dtml-if "_.string.find('f n 0 false no', _.str(_[\'%s\']))>-1"> selected </dtml-if>
 	 >%s          
	 <option value="1"
           <dtml-if "_.string.find('t y 1 true yes', _.str(_[\'%s\']))>-1"> selected </dtml-if>
	 >%s
      </select>
   </TD>
</TR>'''

    insert_booleanBase = '''\t<TR>
<TD ALIGN="RIGHT">%s</TD>
   <TD>
      <select NAME="%s">
         <option value="0">%s
         <option value="1">%s
      </select>
   </TD>
</TR>'''


    __ac_permissions__ = (
	('Use SQL Wizard', ('index_html', 'collectIds', 
			    'selectColumns', 'getObjectsInfo',
			    'createSQLInput', 'quit')),
	)
    meta_type = 'Znolk SQL Wizard'



    dtmlDir = 'Products/ZnolkSQLWizard/'

    def __init__(self, id, connection_id):
	self.id = id
	self.connection_id = connection_id
	self.tables = []
	self.selectedTables = []
	self.tableColumns = []
	self.selectedColumns = []
	self.sqlArguments = []

    def _infoFactory(self):
	connection = getattr(self, self.connection_id)
	exec('import DBInfo.' + connection.database_type)
	return eval('DBInfo.' + connection.database_type + '.Info(connection)')

    def index_html(self):
	"Start input wizard data gathering, show database tables to the user"

	self.info = self._infoFactory()
	self.tables = self.info.getTablesNames() 

	selectTables = HTMLFile( self.dtmlDir + 'selectTables')
	return selectTables(self, self)


    def queryColumns(self,  selectedTables = []):
	"Ask user to select data columns in selected tables"

	if selectedTables != []:
	    self.selectedTables = []
	    for i in selectedTables:
		self.selectedTables.append(self.tables[i])

	    self.tableColumns = []
	    for i, j in self.info.getColumnsNames(self.selectedTables).items():
		self.tableColumns.append(Table(i,j))

	    #print 'Wizard::queryColumns : tableColumns : '

	selectColumns = HTMLFile(self.dtmlDir + 'selectColumns')
	return selectColumns(self,self)
	
    
    def _cleanColName(self, name):
	return string.join(map(string.capitalize, string.split(name, '_')))
    
    def _uniqueCol(self, table, column):
	#return table + string.capitalize(column)
	return table + '.' + column

#PHIL:
    valuelistBase = '''\t<TR>
<TD ALIGN="RIGHT">%s</TD>
   <TD>
      <select NAME="%s">
         <option value=%s> None
         <dtml-in %s>
            <option value="<dtml-var %s>"
               <dtml-try>
                  <dtml-if "_[\'%s\']==_[\'%s\']"> selected </dtml-if>
               <dtml-except>

               </dtml-try>

            > <dtml-var %s>
         </dtml-in>
      </select>
   </TD>
</TR>'''

    insert_valuelistBase = '''\t<TR>
<TD ALIGN="RIGHT">%s</TD>
   <TD>
      <select NAME="%s">
         <option value=%s> None
         <dtml-in %s>
            <option value="<dtml-var %s>"> <dtml-var %s>
         </dtml-in>
      </select>
   </TD>
</TR>'''
    def _buildValuelist(self,table,column,query,pulldown,actual,noneval,isinsert=0):
       #print 'noneval:',noneval
	if isinsert:
    	    return self.insert_valuelistBase  % (
		self._cleanColName(column),
    	        self._uniqueCol(table.tableName,
        			column),

		noneval,
                query,
	        actual,
            	pulldown) + '<dtml-comment>colname:%s,table:%s</dtml-comment>'%(column,table.tableName)
        else:
    	    return self.valuelistBase  % (
		self._cleanColName(column),
    	        self._uniqueCol(table.tableName,
        			column),

		noneval,
                query,
	        actual,
    	        column,
        	actual,
            	pulldown) + '<dtml-comment>colname:%s,table:%s</dtml-comment>'%(column,table.tableName)

        
    def _buildUpdateForm(self, resultPageId,sqlSelectId):
	#base %s parameters: result page id, form contents
	base = """<!--#var standard_html_header-->
<H2><!--#var title_or_id--> </H2>
<dtml-in %s>
<P><FORM ACTION="%s" METHOD="POST">
<TABLE>
%s
</TABLE>
<input type="SUBMIT" name="submit" value="Update"> 
<input type="SUBMIT" name="submit" value="Delete">
<input type=button value="<back" onClick="history.back()">
</FORM>
</dtml-in>
<!--#var standard_html_footer-->"""
        #inputBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><INPUT TYPE="TEXT" NAME="%s" value="<dtml-var %s null="">"></TD></TR>'
        #textAreaBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><textarea NAME="%s" cols=40 rows=4 ><dtml-var %s null=""> </textarea></TD></TR>'
        #labelBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><dtml-var %s><input type="hidden" name=%s value="<dtml-var %s null="">"</TD></TR>' #PHIL for primary keys
	form = []
	for i in self.selectedColumns:
	    form.append('<TR BGCOLOR="#CCCCCC"><TD COLSPAN=2>%s</TD></TR>' 
			% i.tableName)
	    for j in i.columns:
                #PHIL: for pk
                #print '_buildUpdateForm:',i.pkColumns
		if j in i.pkColumns:
		    form.append(self.labelBase 
				% (self._cleanColName(j), 
				   self._uniqueCol(i.tableName, j),
				   self._uniqueCol(i.tableName, j),
				   self._uniqueCol(i.tableName, j)

                                   ))
                #/PHIL
		else:
                        #if j in i.optional:
                        if i.columnControls[j][0]=='edit':
                            form.append(self.inputBase 
                                        % (self._cleanColName(j), 
                                           self._uniqueCol(i.tableName, j),
                                           j
                                           ))
                            
                        if i.columnControls[j][0]=='text':
                            form.append(self.textAreaBase
                                        % (self._cleanColName(j), 
                                           self._uniqueCol(i.tableName, j),
                                           j
                                           ))
                        if i.columnControls[j][0]=='valuelist':
                            params=string.split(i.columnControls[j][1],';')
                            form.append(self._buildValuelist(i,j,params[0],params[1],params[2],params[3]))
                        #else:
                        #    form.append(inputBase 
                        #                % ("<B>" + self._cleanColName(j) + "</B>", 
                        #                   self._uniqueCol(i.tableName, j),
                        #                   j
                        #                   ))

                        if i.columnControls[j][0]=='boolean':
                            try:
                                y,n=string.split(i.columnControls[j][1],';')  #split the extended field
                            except:
                                y,n='Yes','No'                                # if its not filled out correctly take Yes/No
                            
                            form.append(self.booleanBase 
                                        % (self._cleanColName(j), 
                                           self._uniqueCol(i.tableName, j),
                                           j,n, j,y
                                           ))


	return base % (sqlSelectId,resultPageId, string.join(form, '\n'))

    def _buildResultPage(self, sqlMethodId):
	base = """<!--#var standard_html_header-->
<H2><!--#var title_or_id--> <!--#var document_title--></H2>
<P><!--#call "%s(REQUEST)"-->
Your data was inserted in the Database!
<!--#var standard_html_footer-->""" # How do I catch SQL errors?
        return base % sqlMethodId

    def _buildInputForm(self, resultPageId):
	#base %s parameters: result page id, form contents
	base = """<!--#var standard_html_header-->
<H2><!--#var title_or_id--> </H2>
<P><FORM ACTION="%s" METHOD="POST">
<TABLE>
%s
</TABLE>
<input type="SUBMIT" name="submit" value="Insert">
<input type=button value="<back" onClick="history.back()">

</FORM>
<!--#var standard_html_footer-->"""
        #inputBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><INPUT TYPE="TEXT" NAME="%s"></TD></TR>'
        #textAreaBase = '\t<TR><TD ALIGN="RIGHT">%s</TD><TD><textarea NAME="%s" cols=40 rows=4> </textarea></TD></TR>'
	form = []
	for i in self.selectedColumns:
	    form.append('<TR BGCOLOR="#CCCCCC"><TD COLSPAN=2>%s</TD></TR>' 
			% i.tableName)
	    for j in i.columns:
		#if j in i.optional:
                if i.columnControls[j][0]=='edit':
                    form.append(self.insert_inputBase 
                            % (self._cleanColName(j), 
                               self._uniqueCol(i.tableName, j)))
                if i.columnControls[j][0]=='text':
                    form.append(self.insert_textAreaBase 
                            % (self._cleanColName(j), 
                               self._uniqueCol(i.tableName, j)))
                if i.columnControls[j][0]=='valuelist':
                    params=string.split(i.columnControls[j][1],';')
                    form.append(self._buildValuelist(i,j,params[0],params[1],params[2],params[3],1))

                    
		#else:
		#    form.append(inputBase 
		#		% ("<B>" + self._cleanColName(j) + "</B>", 
		#		   self._uniqueCol(i.tableName, j)))

                if i.columnControls[j][0]=='boolean':
                    try:
                        y,n=string.split(i.columnControls[j][1],';')  #split the extended field
                    except:
                        y,n='Yes','No'                                # if its not filled out correctly take Yes/No
                        
                    form.append(self.insert_booleanBase 
                                % (self._cleanColName(j), 
                                   self._uniqueCol(i.tableName, j),
                                   n,y
                                   ))



	return base % (resultPageId, string.join(form, '\n'))

    def _buildResultPage(self, sqlInsertId,sqlUpdateId,sqlDeleteId):
	base = """<!--#var standard_html_header-->
<H2><!--#var title_or_id--> </H2>

<dtml-if "submit=='Insert'">
   <dtml-call "%s(REQUEST)">
   Your data was inserted!
</dtml-if>

<dtml-if "submit=='Update'">
   <dtml-call "%s(REQUEST)">
   Your data was updated!
</dtml-if>

<dtml-if "submit=='Delete'">
   <dtml-call "%s(REQUEST)">
   Your data was deleted!
</dtml-if>

<form>
<input type=button value="<back" onClick="history.back();history.back()">
</form>
<!--#var standard_html_footer-->""" # How do I catch SQL errors?
        return base % (sqlInsertId,sqlUpdateId,sqlDeleteId)


    def debug(self):
	"sss"
	x = ''
	for i in table.columns: x= x+ ' ' + i
	x = x+ '\n************\n'
	for i in table.optional: x= x+ ' ' + i
	return x

    #PHIL: orig def _buildSQLValues(self, table, nonOptCol):
    def _buildSQLValues(self, table):
	template = '<!--#sqlvar %s type=%s %s-->'
	args = []
	for i in table.columns:
	    tp = self.info.type(table.tableName, i)
	    opt = self.info.nullable(table.tableName, i)
	    uniqueCol = self._uniqueCol(table.tableName, i)
	    
	    oneArg = template % (uniqueCol, tp, opt) + ','

	    #PHIL: check for PK and skipPKs

	    if i in table.pkColumns and table.skipPKs:
                pass
            else:
        	if i in table.optional:
                    args.append('<!--#if %s-->\n\t\t\t%s\n\t\t<!--#/if-->'
                                % (uniqueCol, oneArg))
                else:
                    args.append(oneArg)

	#there's just missing the non optional (without the comma)

        #PHIL: removed the nonOptCol stuff
	#tp = self.info.type(table.tableName, nonOptCol)
	#opt = self.info.nullable(table.tableName, nonOptCol)
	#uniqueCol = self._uniqueCol(table.tableName, nonOptCol)
	#args.append(template % (uniqueCol, tp, opt))

	#PHIL: remove last comma:
        #orig: return string.join(args, '\n\t\t')
        res=string.join(args, '\n\t\t')
        pos=string.rfind(res,',')
        res=res[:pos]+res[pos+1:]
        return res
        #/PHIL
        
        

    #PHIL:orig def _buildSQLInto(self, table, nonOptCol):
    def _buildSQLInto(self, table):
	tempCols = []
	for i in table.columns:
	    tempCols.append(i + ',')

	newCol = []
	for i in range(len(table.columns)):
            if table.columns[i] in table.pkColumns and table.skipPKs:
                pass
            else:
                if  table.columns[i] in table.optional:
                    newCol.append('<!--#if %s -->\n\t\t%s\n\t<!--#/if-->' % 
                                  (self._uniqueCol(table.tableName,
					       table.columns[i]), 
                                   tempCols[i]))
                else:
                    newCol.append(tempCols[i])

	#PHIL: orig newCol.append(nonOptCol)
	#PHIL:orig return string.join(newCol,'\n\t')
	#remove last comma
	res=string.join(newCol,'\n\t')
        pos=string.rfind(res,',')
        res=res[:pos]+res[pos+1:]
        return res
        #/PHIL
	
    def _buildSQLInputCommand(self):
	baseCommand = "insert into %s \n\t(%s)\n\tVALUES (%s)"
	command = []
	inputArgs = []

	for i in self.selectedColumns:
	    #building sql method arguments
	    for j in i.columns:
		tempArg = self._uniqueCol(i.tableName, j)
		if j in i.optional:
		    tempArg = tempArg + '=""'
		inputArgs.append(tempArg)

            #PHIL: removed the nonOptCol stuff
	    #Let's find a column that isn't optional, 
	    #I've already verified in getObjectsInfo that it exists
	    #nonOptCol=i.columns[0]
	    #for col in i.columns:
	    #	if col not in i.optional:
		#    nonOptCol = col
		#    i.columns.remove(col)
		#    break
		
#	    command.append(baseCommand % (i.tableName, 
#					  self._buildSQLInto(i, nonOptCol),
#					  self._buildSQLValues(i, nonOptCol)))

	    #building sql method
	    command.append(baseCommand % (i.tableName, 
					  self._buildSQLInto(i),
					  self._buildSQLValues(i)))

	return string.join(inputArgs), string.join(command,'\n<!--#var sql_delimiter -->\n')
    
#PHIL:
    def _buildSQLUpdates(self, table):
	template = '<!--#sqlvar %s type=%s %s-->'

	updates = []
	#print 'pkColumns:',table.pkColumns
	for i in range(len(table.columns)):
	    tp = self.info.type(table.tableName, table.columns[i])
	    opt = self.info.nullable(table.tableName, table.columns[i])
            #print 'column:', table.columns[i],tp,opt
            
	    if not table.columns[i] in table.pkColumns:
                if table.columns[i] in table.optional:
        		updates.append('<!--#if %s -->\n\t\t%s=<dtml-sqlvar %s type=%s %s>,\n\t<!--#/if-->' % 
                                  (self._uniqueCol(table.tableName,
                                		       table.columns[i]), 
                                   #self._uniqueCol(table.tableName,
                                	#	       table.columns[i]),
                                   table.columns[i],
                                   self._uniqueCol(table.tableName,
                                        	       table.columns[i]),
                                   tp,
                                   opt
                                   ))
        	else:
        		updates.append('%s=<dtml-sqlvar %s type=%s %s>,' % 
                                   (
                                    #self._uniqueCol(table.tableName,
                                		   #    table.columns[i]),
                                   table.columns[i], 
                                   self._uniqueCol(table.tableName,
                                        	       table.columns[i]),
                                   tp,
                                   opt
                                   ))
                    
	    #else:
                #print 'pkcolumn:', table.columns[i]


	#newCol.append(nonOptCol)

        #remove last ','
	res=string.join(updates,'\n\t')
	pos=string.rfind(res,',')
	res=res[:pos]+res[pos+1:]
	return res
	

    def _buildSQLWhere(self, table):
	updates = []
	for i in range(len(table.columns)):
	    tp = self.info.type(table.tableName, table.columns[i])
	    if table.columns[i] in table.pkColumns:
		updates.append(' %s=<dtml-sqlvar %s type=%s> ' % 
			      (self._uniqueCol(table.tableName,
					       table.columns[i]),
                               self._uniqueCol(table.tableName,
					       table.columns[i]),
                               tp
                               ))
	return string.join(updates,' and ')

    def _buildSQLPkParams(self, table):
	updates = []
	for i in range(len(table.columns)):
	    tp = self.info.type(table.tableName, table.columns[i])
	    if table.columns[i] in table.pkColumns:
		updates.append('%s=<dtml-var %s>' % 
			      (self._uniqueCol(table.tableName,
					       table.columns[i]), 
                               table.columns[i]
                               ))
	return string.join(updates,'&')


    def _buildSQLUpdateCommand(self):
	baseCommand = "update %s set \n\t%s\n\t where %s"
	command = []
	inputArgs = []

	for i in self.selectedColumns:
            #print 'cols:',i.columns
	    #building sql method arguments
	    for j in i.columns:
                #print 'col:',j
		tempArg = self._uniqueCol(i.tableName, j)
		if j in i.optional:
		    tempArg = tempArg + '=""'
		inputArgs.append(tempArg)

	    #Let's find a column that isn't optional, 
	    #I've already verified in getObjectsInfo that it exists
	    #for col in i.columns:
		#if col not in i.optional:
		#    nonOptCol = col
		#    i.columns.remove(col)
		#    break

	    #building sql method
	    command.append(baseCommand % (i.tableName, 
					  self._buildSQLUpdates(i),
					  self._buildSQLWhere(i)))

	return string.join(inputArgs), string.join(command,'\n<!--#var sql_delimiter -->\n')

    def _buildSQLDeleteCommand(self):
	baseCommand = "delete from %s  where %s"
	command = []
	inputArgs = []

	for i in self.selectedColumns:
            #print 'cols:',i.columns
	    #building sql method arguments
	    for j in i.pkColumns:
                #print 'col:',j
		tempArg = self._uniqueCol(i.tableName, j)
		inputArgs.append(tempArg)

	    #building sql method
	    command.append(baseCommand % (i.tableName, 
					  self._buildSQLWhere(i)))

	return string.join(inputArgs), string.join(command,'\n<!--#var sql_delimiter -->\n')

    def _buildSQLSelectCommand(self):
	baseCommand = "select * from %s  where %s"
	command = []
	inputArgs = []

	for i in self.selectedColumns:
            #print 'cols:',i.columns
	    #building sql method arguments
	    for j in i.columns:
                #print 'col:',j
		tempArg = self._uniqueCol(i.tableName, j)
		if j in i.optional:
		    tempArg = tempArg + '=""'
		if j in i.pkColumns: inputArgs.append(tempArg)

	    #Let's find a column that isn't optional, 
	    #I've already verified in getObjectsInfo that it exists
	    #for col in i.columns:
		#if col not in i.optional:
		#    nonOptCol = col
		#    i.columns.remove(col)
		#    break

	    #building sql method
	    command.append(baseCommand % (i.tableName, 
					  self._buildSQLWhere(i)))

	return string.join(inputArgs), string.join(command,'\n<!--#var sql_delimiter -->\n')

    def _buildSQLBrowseCommand(self):
	baseCommand = "select * from %s  "
	command = []
	inputArgs = []

	for i in self.selectedColumns:
            #print 'cols:',i.columns
	    #building sql method arguments

	    #Let's find a column that isn't optional, 
	    #I've already verified in getObjectsInfo that it exists
	    #for col in i.columns:
		#if col not in i.optional:
		#    nonOptCol = col
		#    i.columns.remove(col)
		#    break

	    #building sql method
	    command.append(baseCommand % (i.tableName, 
					  ))

	return string.join(inputArgs), string.join(command,'\n<!--#var sql_delimiter -->\n')

    def _buildBrowsePage(self, sqlMethod,resultPageId,insertFormId):
	#base %s parameters: result page id, form contents
	base = """<dtml-var standard_html_header>
	
<h2><dtml-var title_or_id></h2>	
<dtml-let QUERY_GOAL=URL>
<dtml-in %s size=25 start=query_start>
   <dtml-if sequence-start>


      <dtml-if previous-sequence>

        <a href="<dtml-var QUERY_GOAL><dtml-var sequence-query
                 >query_start=<dtml-var
                 previous-sequence-start-number>">
        (Previous <dtml-var previous-sequence-size> results)
        </a>

      </dtml-if previous-sequence>

      <table border>
        <tr>
          %s
        </tr>
      
   </dtml-if sequence-start>

        <tr>
          %s
        </tr>

   <dtml-if sequence-end>

      </table>
      <dtml-if next-sequence>

         <a href="<dtml-var QUERY_GOAL><dtml-var sequence-query
            >query_start=<dtml-var
            next-sequence-start-number>">
         (Next <dtml-var next-sequence-size> results)
         </a>

      </dtml-if next-sequence>
   </dtml-if sequence-end>

<dtml-else>

  There was no data matching this <dtml-var title_or_id> query.

</dtml-in>

<form action=%s >
<input name=new value="create new" type=submit>
</form>

<form action=<dtml-var QUERY_GOAL>>
<input type=submit value="refresh">
</form>
</dtml-let>
<dtml-var standard_html_footer>"""
	
        headBase = '<TH ALIGN="RIGHT"> %s</TH>'
        rowBase = '<TD ALIGN="RIGHT"><dtml-var %s null=""></TD>'
        linkBase = '<TD ALIGN="RIGHT"><a href=%s?%s>edit</a></TD>'
	head = []
	row = []
	for i in self.selectedColumns:
	    #style='<TR BGCOLOR="#CCCCCC"><TD COLSPAN=2>%s</TD></TR>' % (i.tableName)
	    for j in i.columns:
               head.append(headBase 
			    % ("<B>" + self._cleanColName(j) + "</B>"))
               
               row.append(rowBase 
			    % (j))
            row.append(linkBase 
		        % (resultPageId,self._buildSQLPkParams(i)))


        #print 'head:',head
        #print 'row:',row
	return base % (sqlMethod, string.join(head, '\n'),string.join(row, '\n'),insertFormId)
   
#/PHIL

    def getObjectsInfo(self, REQUEST):
	"Store data and Collect id's and title of objects to be created"
	self.selectedColumns = []
	skipPKs=0
	for i in self.selectedTables:
            
            
	    if REQUEST.has_key(i):
                if REQUEST.has_key('skipPKs.'+i):
                   skipPKs=1
                   #print 'skip PKS'
		optional = []
		if REQUEST.has_key('opt.' + i):
		    optional = REQUEST['opt.' + i]
		    #print 'optional:',optional

                #PHIL: select for primary keys
	        pkColumns=[]
		if REQUEST.has_key('pk.' + i):
		    pkColumns = REQUEST['pk.' + i]
		    #print 'Table::getObjectsInfo:',pkColumns

                #get edit controls
	        controls=[]
		if REQUEST.has_key('control.' + i):
		    controls = REQUEST['control.' + i]
		    #print 'Table::getObjectsInfo:',controls

	        extensions=[]
		if REQUEST.has_key('extension.' + i):
		    extensions = REQUEST['extension.' + i]
		    #print 'Table::getObjectsInfo extensions:',extensions

                #print 'TableColumns:',self.tableColumns[0].columns
		#/PHIL

		oneColNonOptional = None
		for j in REQUEST[i]:
		    if j not in optional:
			oneColNonOptional = 1
			break
		if oneColNonOptional is None:
		    return MessageDialog(title="Error",
					 message = "<h1>Error</h1>There must be at least one selected non-optional column",
					 action = REQUEST['HTTP_REFERER']
					 )
		
#PHIL:orig		self.selectedColumns.append(Table(i, REQUEST[i], 
#						  optional))
                #print 'pkColumns1:',pkColumns
		self.selectedColumns.append(Table(i, REQUEST[i], 
						  optional,pkColumns,skipPKs=skipPKs,controls=controls,extensions=extensions,allColumns=self.tableColumns[0].columns))

	if self.selectedColumns == []:
	    return MessageDialog(title="Error",
				 message = "<h1>Error</h1>You Must select at least one column of one table",
				 action = REQUEST['HTTP_REFERER']
				 )

	allTables = string.join(map(string.capitalize, self.selectedTables),'')
	defaultTitles = string.join(map(string.capitalize, 
					self.selectedTables))
	formDefaultId = _generateId(self.aq_parent, allTables + "InsertForm")
	updateFormDefaultId = _generateId(self.aq_parent, allTables + "UpdateForm") #PHIL
	resultDefaultId = _generateId(self.aq_parent, allTables + "Result")

	sqlMethodDefaultId = _generateId(self.aq_parent, 
					 allTables + "InsertMethod")

	sqlUpdateMethodDefaultId = _generateId(self.aq_parent,      #PHIL
					 allTables + "UpdateMethod")
	sqlSelectMethodDefaultId = _generateId(self.aq_parent,      #PHIL
					 allTables + "PkSelectMethod")
	sqlBrowseMethodDefaultId = _generateId(self.aq_parent,      #PHIL
					 allTables + "BrowseMethod")
	browseReportDefaultId = _generateId(self.aq_parent,      #PHIL
					 allTables + "BrowseReport")
	sqlDeleteMethodDefaultId = _generateId(self.aq_parent,      #PHIL
					 allTables + "DeleteMethod")

	collectIds = HTMLFile(self.dtmlDir + 'collectIds')

	return  collectIds(self, 
			   formId=formDefaultId,
                           updateFormId=updateFormDefaultId, #PHIL
			   resultId=resultDefaultId,
			   sqlId=sqlMethodDefaultId,
			   sqlUpdateId=sqlUpdateMethodDefaultId,#PHIL
			   sqlSelectId=sqlSelectMethodDefaultId,#PHIL
			   sqlBrowseMethodId=sqlBrowseMethodDefaultId,#PHIL
			   browseReportId=browseReportDefaultId,#PHIL
			   sqlDeleteId=sqlDeleteMethodDefaultId,#PHIL
			   titles=defaultTitles)

#PHIL: orig:    def createSQLInput(self, RESPONSE, URL2, formId, resultId, sqlId, 
#		       formTitle='', resultTitle = '', sqlTitle = ''):
    
    def createSQLInput(self, RESPONSE, URL2, formId,updateFormId, resultId, sqlId,
                       sqlUpdateId ,sqlSelectId, sqlBrowseMethodId,browseReportId,sqlDeleteId,
		       formTitle='',updateFormTitle='', resultTitle = '',  sqlTitle = '',
                       sqlUpdateTitle='',sqlSelectTitle='',sqlBrowseMethodTitle='',browseReportTitle='',sqlDeleteTitle=''):
	"""Method to add the Z SQL Method and a Form to access it, 
	also delete Wizard from parent folder """

	#PHIL:
	#print 'Table::createSQLInput:'

	try:
	    self.aq_parent.manage_addDTMLDocument(formId, formTitle, 
						  self._buildInputForm(resultId) ) 
	except AttributeError:
	    self.aq_parent.manage_addDocument(formId, formTitle, 
                                              self._buildInputForm(resultId) ) 
	#PHIL:added updateForm
	#print 'Table::createSQLInput:',updateFormId,updateFormTitle
	try:
	    self.aq_parent.manage_addDTMLDocument(updateFormId, updateFormTitle, 
						  self._buildUpdateForm(resultId,sqlSelectId) ) 
	except AttributeError:
	    self.aq_parent.manage_addDocument(updateFormId, updateFormTitle, 
        						  self._buildUpdateForm(resultId,sqlSelectId) ) 
        #/PHIL
	    
	try:
	    self.aq_parent.manage_addDTMLDocument(resultId, resultTitle, 
						  self._buildResultPage(sqlId,sqlUpdateId,sqlDeleteId) ) 
	except AttributeError:
	    self.aq_parent.manage_addDocument(resultId, resultTitle, 
						  self._buildResultPage(sqlId,sqlUpdateId,sqlDeleteId) ) 

        #PHIL updateResult Page	    
	#try:
	#    self.aq_parent.manage_addDTMLDocument(updateResultId, updateResultTitle, 
	#					  self._buildResultPage(sqlUpdateId) ) 
	#except AttributeError:
	#    self.aq_parent.manage_addDocument(updateResultId, updateResultTitle, 
	#					  self._buildResultPage(sqlUpdateId) )
	#/PHIL

	args, command = self._buildSQLInputCommand()

	#PHIL:
	#print args,command
	

	self.aq_parent._setObject(sqlId, SQL(sqlId, sqlTitle,
		self.connection_id, args, command))

        #PHIL build sql update method
	args, command = self._buildSQLUpdateCommand()

	self.aq_parent._setObject(sqlUpdateId, SQL(sqlUpdateId, sqlUpdateTitle,
		self.connection_id, args, command))

	#/PHIL
	
        #PHIL build sql Select Method
	args, command = self._buildSQLSelectCommand()

	self.aq_parent._setObject(sqlSelectId, SQL(sqlSelectId, sqlSelectTitle,
		self.connection_id, args, command))

	#/PHIL

        #PHIL build sql Browse Method
	args, command = self._buildSQLBrowseCommand()

	#print args,command
	

	self.aq_parent._setObject(sqlBrowseMethodId, SQL(sqlBrowseMethodId, sqlBrowseMethodTitle,
		self.connection_id, args, command))

	#/PHIL
        #PHIL	    build Browse Page
	try:
	    self.aq_parent.manage_addDTMLDocument(browseReportId, browseReportTitle, 
						  self._buildBrowsePage(sqlBrowseMethodId,updateFormId,formId) ) 
	except AttributeError:
	    self.aq_parent.manage_addDocument(browseReportId, browseReportTitle, 
						  self._buildBrowsePage(sqlBrowseMethodId,updateFormId,formId) )
	#/PHIL

        #PHIL build sql delete method
	args, command = self._buildSQLDeleteCommand()

	self.aq_parent._setObject(sqlDeleteId, SQL(sqlDeleteId, sqlDeleteTitle,
		self.connection_id, args, command))

	#/PHIL

        #PHIL build delete result page
	#try:
	#    self.aq_parent.manage_addDTMLDocument(deleteResultId, deleteResultTitle, 
	#					  self._buildResultPage(sqlDeleteId) ) 
	#except AttributeError:
	#    self.aq_parent.manage_addDocument(deleteResultId, deleteResultTitle, 
	#					  self._buildResultPage(sqlDeleteId) )
	#/PHIL
	self.quit(RESPONSE, URL2)

    def quit(self, RESPONSE, URL2):
	"Cleans temporary objects used by P SQL Wizard"
	
	delattr(self.aq_parent, self.id) #deleting wizard, if you leave in the middle there will be trash in your folder
	
	
	RESPONSE.redirect(URL2 + '/manage_main') #should I show some message?


    

